﻿// $Id: HorizontalLayout.cs 78 2012-08-21 20:31:13Z mawerick $
//
//  Copyright (C) Mawerick, WrongPlace.Net 2005 - 2012
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.Layout;

namespace Twp.Controls
{
	public enum HorizontalLayoutStyle
	{
		LeftToRight,
		RightToLeft,
		Fill,
	}

	public class HorizontalLayout : LayoutEngine
    {
        #region Properties

        /// <summary>
        /// Returns a value equal to the full width used by the controls in the container.
        /// </summary>
        public int Width
        {
            get { return this.width; }
        }
        private int width = 0;

        /// <summary>
        /// Gets or sets a value representing the width of the space between controls.
        /// </summary>
        /// <remarks>
        /// If set to -1, the layout will use each control's right margin for spacing. This is the default.
        /// </remarks>
        public int Spacing
        {
            get { return this.spacing; }
            set { this.spacing = value; }
        }
        private int spacing = -1;

        /// <summary>
        /// Gets or sets a value indicating whether or not hidden controls are included by the LayoutEngine.
        /// </summary>
        public bool IncludeHidden
        {
            get { return this.includeHidden; }
            set { this.includeHidden = value; }
        }
        private bool includeHidden = false;

        /// <summary>
        /// Gets or sets a value representing the style used for the layout.
        /// </summary>
        public HorizontalLayoutStyle LayoutStyle
        {
        	get { return this.layoutStyle; }
        	set { this.layoutStyle = value; }
        }
        private HorizontalLayoutStyle layoutStyle = HorizontalLayoutStyle.LeftToRight;
        
        #endregion

        #region Methods

        public override bool Layout( object container, System.Windows.Forms.LayoutEventArgs layoutEventArgs )
        {
            Control parent = container as Control;
            
            switch( this.layoutStyle )
            {
            	case HorizontalLayoutStyle.LeftToRight:
            		this.LayoutLeftToRight( parent );
            		break;
            		
            	case HorizontalLayoutStyle.RightToLeft:
            		this.LayoutRightToLeft( parent );
            		break;
            		
            	case HorizontalLayoutStyle.Fill:
            		break;
            		
            	default:
            		throw new Exception( "Invalid HorizontalLayoutStyle value: " + this.layoutStyle.ToString() );
            }
            
            return true;
        }
        
        private void LayoutLeftToRight( Control parent )
        {
            Rectangle pDisplayRect = parent.DisplayRectangle;
            Point nextLocation = pDisplayRect.Location;
            nextLocation.Offset( parent.Margin.Left, parent.Margin.Top );

            foreach( Control control in parent.Controls )
            {
                if( !control.Visible && !this.includeHidden )
                    continue;

                control.Location = nextLocation;
                control.Height = pDisplayRect.Height - parent.Margin.Vertical;

                if( this.spacing >= 0 )
                    nextLocation.Offset( control.Width + this.spacing, 0 );
                else
                    nextLocation.Offset( control.Width + control.Margin.Right, 0 );
            }

            this.width = nextLocation.X + parent.Margin.Right;
        }
        
        private void LayoutRightToLeft( Control parent )
        {
            Rectangle pDisplayRect = parent.DisplayRectangle;
            Point nextLocation = pDisplayRect.Location;
            nextLocation.Offset( pDisplayRect.Width, 0 );
            nextLocation.Offset( -parent.Margin.Right, parent.Margin.Top );

            foreach( Control control in parent.Controls )
            {
                if( !control.Visible && !this.includeHidden )
                    continue;

                if( this.spacing >= 0 )
                	nextLocation.Offset( -(control.Width + this.spacing), 0 );
                else
                	nextLocation.Offset( -(control.Width + control.Margin.Right), 0 );

                control.Location = nextLocation;
                control.Height = pDisplayRect.Height - parent.Margin.Vertical;
            }

            this.width = pDisplayRect.Width - (nextLocation.X + parent.Margin.Left);
        }
        
        private void LayoutFill( Control parent )
        {
            Rectangle pDisplayRect = parent.DisplayRectangle;
            Point nextLocation = pDisplayRect.Location;
            nextLocation.Offset( parent.Margin.Left, parent.Margin.Top );

            foreach( Control control in parent.Controls )
            {
                if( !control.Visible && !this.includeHidden )
                    continue;

                control.Location = nextLocation;
                control.Height = pDisplayRect.Height - parent.Margin.Vertical;

                if( this.spacing >= 0 )
                    nextLocation.Offset( control.Width + this.spacing, 0 );
                else
                    nextLocation.Offset( control.Width + control.Margin.Right, 0 );
            }

            this.width = nextLocation.X + parent.Margin.Right;
        }

        #endregion
    }
}
