﻿// $Id: Line.cs 73 2012-04-11 01:37:42Z mawerick $
//
//  Copyright (C) Mawerick, WrongPlace.Net 2009 - 2012
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Twp.Controls
{
	/// <summary>
	/// Specifies the style used to paint a line.
	/// </summary>
	public enum LineStyle
	{
		/// <summary>The line has a sunken look.</summary>
		Sunken,
		/// <summary>The line has a raised look.</summary>
		Raised,
		/// <summary>The line has a flat look.</summary>
		Flat,
	}

	/// <summary>
	/// Description of Line.
	/// </summary>
    [Description( "Provides a simple horizontal or vertical line. Purely cosmetic." )]
    [Designer( typeof( Design.LineDesigner ), typeof( System.ComponentModel.Design.IDesigner ) )]
    [ToolboxBitmap( typeof( Line ), "Resources.Line.bmp" )]
    [DefaultProperty( "LineStyle" )]
    [DefaultEvent( "OrientationChanged" )]
    public class Line : Control, INotifyPropertyChanged
	{
        /// <summary>
        /// Initializes a new instance of the Line class with default settings.
        /// </summary>
		public Line() : base()
		{
            this.SetStyle( ControlStyles.AllPaintingInWmPaint |
                 ControlStyles.ResizeRedraw |
                 ControlStyles.DoubleBuffer |
                 ControlStyles.SupportsTransparentBackColor,
                 true );
			this.BackColor = Color.Transparent;
			this.TabStop = false;
		}

		public event PropertyChangedEventHandler PropertyChanged;

		protected virtual void OnPropertyChanged( PropertyChangedEventArgs e )
		{
			if( this.PropertyChanged != null )
				this.PropertyChanged( this, e );
		}

		/// <summary>
		/// Gets or sets a value representing the orientation of the line.
		/// The default is Horizontal.
		/// </summary>
		[Category( "Appearance" )]
		[Description( "The orientation of the line." )]
		[DefaultValue( Orientation.Horizontal )]
		public Orientation Orientation
		{
			get { return this.orientation; }
			set
			{
				if( this.orientation != value )
				{
					this.orientation = value;
					if( this.orientation == Orientation.Horizontal )
					{
						if( this.Width < this.Height )
						{
							int temp = this.Width;
							this.Width = this.Height;
							this.Height = temp;
						}
					}
					else
					{
						if( this.Width > this.Height )
						{
							int temp = this.Width;
							this.Width = this.Height;
							this.Height = temp;
						}
					}

					this.Size = FitSize;
					this.OnPropertyChanged( new PropertyChangedEventArgs( "Orientation" ) );
				}
			}
		}
		private Orientation orientation = Orientation.Horizontal;

		public new Size Size
		{
			get { return base.Size; }
			set
			{
				if( this.orientation == Orientation.Horizontal )
				{
					this.Width = value.Width;
					this.Height = FitSize.Height;
				}
				else
				{
					this.Width = FitSize.Width;
					this.Height = value.Height;
				}
			}
		}

		private Size FitSize
		{
			get
			{
				if( this.orientation == Orientation.Horizontal )
					return new Size( this.ClientSize.Width, (this.linePadding * 2) + 2 );
				else
					return new Size( ( this.linePadding * 2 ) + 2, this.ClientSize.Height );
			}
		}

		/// <summary>
		/// Gets or sets a value representing the style of the line.
		/// </summary>
		[Category( "Appearance" )]
		[Description( "The style of the line." )]
		[DefaultValue( LineStyle.Sunken )]
		public LineStyle LineStyle
		{
			get { return this.lineStyle; }
			set
			{
				if( this.lineStyle != value )
				{
					this.lineStyle = value;
					this.Invalidate();
					this.OnPropertyChanged( new PropertyChangedEventArgs( "LineStyle" ) );
				}
			}
		}
		private LineStyle lineStyle = LineStyle.Sunken;

		/// <summary>
		/// Gets or sets a value representing the padding on either side of the line.
		/// </summary>
		[Category( "Appearance" )]
		[Description( "The padding in either side of the line" )]
		[DefaultValue( 4 )]
		public int LinePadding
		{
			get { return this.linePadding; }
			set
			{
				if( this.linePadding != value )
				{
					this.linePadding = value;
					this.Size = FitSize;
					this.OnPropertyChanged( new PropertyChangedEventArgs( "LinePadding" ) );
				}
			}
		}
		private int linePadding = 4;

		/// <summary>
		/// Raises the <see cref="System.Windows.Forms.Control.Paint"/> event.
		/// </summary>
		/// <param name="e"></param>
		protected override void OnPaint( PaintEventArgs e )
		{
			Point start = Point.Empty;
			Point end = Point.Empty;
			Color light = Color.Empty;
			Color dark = Color.Empty;
			switch( this.lineStyle )
			{
				case LineStyle.Sunken:
                    light = SystemColors.ButtonHighlight;
                    dark = SystemColors.ButtonShadow;
					break;
				case LineStyle.Raised:
                    light = SystemColors.ButtonShadow;
                    dark = SystemColors.ButtonHighlight;
					break;
				case LineStyle.Flat:
					// light remains empty, only dark gets rendered.
                    dark = SystemColors.ButtonShadow;
					break;
				default:
					return;
			}
			Rectangle rect = this.ClientRectangle;
			switch( this.orientation )
			{
				case Orientation.Horizontal:
					start = new Point( rect.X, rect.Y + this.linePadding );
					end = new Point( rect.X + rect.Width, rect.Y + this.linePadding );
					e.Graphics.DrawLine( new Pen( dark ), start, end );
					if( light != Color.Empty )
					{
						start.Offset( 0, 1 );
						end.Offset( 0, 1 );
						e.Graphics.DrawLine( new Pen( light ), start, end );
					}
					break;
				case Orientation.Vertical:
					start = new Point( rect.X + this.linePadding, rect.Y );
					end = new Point( rect.X + this.linePadding, rect.Y + rect.Height );
					e.Graphics.DrawLine( new Pen( dark ), start, end );
					if( light != Color.Empty )
					{
						start.Offset( 1, 0 );
						end.Offset( 1, 0 );
						e.Graphics.DrawLine( new Pen( light ), start, end );
					}
					break;
			}
		}

		protected override void OnResize( EventArgs e )
		{
			this.Invalidate();
		}
	}
}
