﻿// $Id: Popup.cs 70 2012-04-03 03:14:28Z mawerick $
//
//  Copyright (C) Mawerick, WrongPlace.Net 2009 - 2012
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using System.Windows.Forms;
using Twp.Utilities;

namespace Twp.Controls
{
    public enum PopupDirection
    {
        Down,
        Left,
        Right,
        Up,
        DownLeft,
        DownRight,
        UpLeft,
        UpRight,
    }

    public class Popup : ToolStripDropDown
    {
        #region Properties

        public Control Content
        {
            get { return this.content; }
        }
        private Control content;

        private ToolStripControlHost host;
        private Popup ownerPopup;
        private Popup childPopup;

        public bool Resizable
        {
            get { return (this.resizable && !this.resizeLocked); }
            set
            {
                if( this.resizable != value )
                {
                    this.resizable = value;
                }
            }
        }
        private bool resizable = false;
        private bool resizeLocked = false;
        private bool resizableTop = false;
        private bool resizableLeft = false;

        public new Size MinimumSize
        {
            get { return this.minimumSize; }
            set
            {
                if( this.minimumSize != value )
                    this.minimumSize = value;
            }
        }
        private Size minimumSize;

        public new Size MaximumSize
        {
            get { return this.maximumSize; }
            set
            {
                if( this.maximumSize != value )
                    this.maximumSize = value;
            }
        }
        private Size maximumSize;

        public PopupDirection Direction
        {
            get { return this.direction; }
            set
            {
                if( this.direction != value )
                    this.direction = value;
            }
        }
        private PopupDirection direction;

        protected override CreateParams CreateParams
        {
            [SecurityPermission( SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode )]
            get
            {
                CreateParams cp = base.CreateParams;
                cp.ExStyle |= NativeMethods.WS_EX_NOACTIVATE;
                return cp;
            }
        }

        #endregion

        #region Constructor

        public Popup( Control content )
        {
            if( content == null )
                throw new ArgumentNullException( "content" );

            this.content = content;
            this.AutoSize = false;
            this.DoubleBuffered = true;
            this.ResizeRedraw = true;

            this.host = new ToolStripControlHost( this.content );

            Size contentSize = new Size( this.content.Width - 3, this.content.Height - 4 );
            this.minimumSize = this.content.MinimumSize;
            this.maximumSize = this.content.MaximumSize;
            this.Size = this.content.Size;

            this.content.MinimumSize = contentSize;
            this.content.MaximumSize = contentSize;
            this.content.Size = contentSize;

            this.content.Location = Point.Empty;
            this.Items.Add( host );

            this.content.Disposed += new EventHandler( this.OnContentDisposed );
            this.content.RegionChanged += new EventHandler( this.OnContentRegionChanged );
            this.content.Paint += new PaintEventHandler( this.OnContentPaint );

            this.UpdateRegion();
        }

        #endregion

        #region Methods

        private void OnContentDisposed( object sender, EventArgs e )
        {
            this.content = null;
            this.Dispose( true );
        }

        private void OnContentRegionChanged( object sender, EventArgs e )
        {
            this.UpdateRegion();
        }

        private void OnContentPaint( object sender, PaintEventArgs e )
        {
            this.PaintSizeGrip( e );
        }

        protected void UpdateRegion()
        {
            if( this.Region != null )
            {
                this.Region.Dispose();
                this.Region = null;
            }
            if( this.content.Region != null )
            {
                this.Region = this.content.Region.Clone();
            }
        }

        public void Show( Control control )
        {
            if( control == null )
                throw new ArgumentNullException( "control" );

            this.Show( control, control.ClientRectangle );
        }

        public void Show( Control control, Rectangle rect )
        {
            if( control == null )
                throw new ArgumentNullException( "control" );

            this.SetOwnerItem( control );

            Log.Debug( "[Popup.Show] Content Location: {0} Content Size: {1} Popup Size: {2}", this.content.Location, this.content.Size, this.Size );

            Point location;
            ToolStripDropDownDirection direction;
            int offset = (this.content.Width - rect.Width) / 2;

            switch( this.direction )
            {
                case PopupDirection.Down:
                    location = control.PointToScreen( new Point( rect.Left - offset, rect.Bottom ) );
                    direction = ToolStripDropDownDirection.BelowRight;
                    break;
                case PopupDirection.Left:
                    location = control.PointToScreen( new Point( rect.Left, rect.Top ) );
                    direction = ToolStripDropDownDirection.Left;
                    break;
                case PopupDirection.Right:
                    location = control.PointToScreen( new Point( rect.Right, rect.Top ) );
                    direction = ToolStripDropDownDirection.Right;
                    break;
                case PopupDirection.Up:
                    location = control.PointToScreen( new Point( rect.Left - offset, rect.Top ) );
                    direction = ToolStripDropDownDirection.AboveRight;
                    break;
                case PopupDirection.DownLeft:
                    location = control.PointToScreen( new Point( rect.Right, rect.Bottom ) );
                    direction = ToolStripDropDownDirection.BelowLeft;
                    break;
                case PopupDirection.DownRight:
                    location = control.PointToScreen( new Point( rect.Left, rect.Bottom ) );
                    direction = ToolStripDropDownDirection.BelowRight;
                    break;
                case PopupDirection.UpLeft:
                    location = control.PointToScreen( new Point( rect.Right, rect.Top ) );
                    direction = ToolStripDropDownDirection.AboveLeft;
                    break;
                case PopupDirection.UpRight:
                    location = control.PointToScreen( new Point( rect.Left, rect.Top ) );
                    direction = ToolStripDropDownDirection.AboveRight;
                    break;
                default:
                    throw new Exception( String.Format( "Unknown PopupDirection {0}", this.direction ) );
            }

            Rectangle screen = Screen.FromControl( control ).WorkingArea;
            if( location.X < screen.Left )
            {
                location.X = screen.Left;
            }
            else if( location.X + this.Width > screen.Right )
            {
                this.resizableLeft = true;
                location.X = screen.Right - this.Width;
            }
            if( location.Y < screen.Top )
            {
                location.Y = screen.Top;
            }
            else if( location.Y + this.Height > screen.Bottom )
            {
                this.resizableTop = true;
                location.Y = screen.Bottom - this.Height;
            }
            location = control.PointToClient( location );
            this.Show( control, location, direction );
        }

        private void SetOwnerItem( Control control )
        {
            if( control == null )
                return;
            if( control is Popup )
            {
                Popup parent = control as Popup;
                this.ownerPopup = parent;
                ownerPopup.childPopup = this;
                OwnerItem = parent.Items[0];
                return;
            }
            if( control.Parent != null )
            {
                this.SetOwnerItem( control.Parent );
            }
        }

        protected override void OnSizeChanged( EventArgs e )
        {
            this.content.MinimumSize = this.Size;
            this.content.MaximumSize = this.Size;
            this.content.Size = this.Size;
            this.content.Location = Point.Empty;
            base.OnSizeChanged( e );
        }

        protected override void OnOpening( CancelEventArgs e )
        {
            if( this.content.IsDisposed || this.content.Disposing )
            {
                e.Cancel = true;
                return;
            }
            this.UpdateRegion();
            base.OnOpening( e );
        }

        protected override void OnOpened( EventArgs e )
        {
            if( this.ownerPopup != null )
                this.ownerPopup.resizeLocked = true;
            this.content.Focus();
            base.OnOpened( e );
        }

        protected override void OnClosed( ToolStripDropDownClosedEventArgs e )
        {
            if( this.ownerPopup != null )
                this.ownerPopup.resizeLocked = false;
            base.OnClosed( e );
        }

        #region Resize support

        [SecurityPermission( SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode )]
        protected override void WndProc( ref Message m )
        {
            if( this.InternalProcessResizing( ref m, false ) )
                return;
            base.WndProc( ref m );
        }

        [SecurityPermission( SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode )]
        public bool ProcessResizing( ref Message m )
        {
            return this.InternalProcessResizing( ref m, true );
        }

        [SecurityPermission( SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode )]
        private bool InternalProcessResizing( ref Message m, bool contentControl )
        {
            if( m.Msg == NativeMethods.WM_NCACTIVATE && m.WParam != IntPtr.Zero && this.childPopup != null && this.childPopup.Visible )
                this.childPopup.Hide();
            if( !this.Resizable )
                return false;
            if( m.Msg == NativeMethods.WM_NCHITTEST )
                return this.OnNcHitTest( ref m, contentControl );
            else if( m.Msg == NativeMethods.WM_GETMINMAXINFO )
                return this.OnGetMinMaxInfo( ref m );
            return false;
        }

        [SecurityPermission( SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.UnmanagedCode )]
        private bool OnGetMinMaxInfo( ref Message m )
        {
            NativeMethods.MINMAXINFO minMax = (NativeMethods.MINMAXINFO) Marshal.PtrToStructure( m.LParam, typeof( NativeMethods.MINMAXINFO ) );
            minMax.maxTrackSize = this.MaximumSize;
            minMax.minTrackSize = this.MinimumSize;
            Marshal.StructureToPtr( minMax, m.LParam, true );
            return true;
        }

        private bool OnNcHitTest( ref Message m, bool contentControl )
        {
            int x = NativeMethods.LOWORD( m.LParam );
            int y = NativeMethods.HIWORD( m.LParam );
            Point clientLocation = this.PointToClient( new Point( x, y ) );

            GripBounds gripBounds = new GripBounds( contentControl ? content.ClientRectangle : this.ClientRectangle );
            IntPtr transparent = new IntPtr( NativeMethods.HTTRANSPARENT );

            if( this.resizableTop )
            {
                if( this.resizableLeft && gripBounds.TopLeft.Contains( clientLocation ) )
                {
                    m.Result = contentControl ? transparent : (IntPtr) NativeMethods.HTTOPLEFT;
                    return true;
                }
                if( !this.resizableLeft && gripBounds.TopRight.Contains( clientLocation ) )
                {
                    m.Result = contentControl ? transparent : (IntPtr) NativeMethods.HTTOPRIGHT;
                    return true;
                }
                if( gripBounds.Top.Contains( clientLocation ) )
                {
                    m.Result = contentControl ? transparent : (IntPtr) NativeMethods.HTTOP;
                    return true;
                }
            }
            else
            {
                if( this.resizableLeft && gripBounds.BottomLeft.Contains( clientLocation ) )
                {
                    m.Result = contentControl ? transparent : (IntPtr) NativeMethods.HTBOTTOMLEFT;
                    return true;
                }
                if( !this.resizableLeft && gripBounds.BottomRight.Contains( clientLocation ) )
                {
                    m.Result = contentControl ? transparent : (IntPtr) NativeMethods.HTBOTTOMRIGHT;
                    return true;
                }
                if( gripBounds.Bottom.Contains( clientLocation ) )
                {
                    m.Result = contentControl ? transparent : (IntPtr) NativeMethods.HTBOTTOM;
                    return true;
                }
            }
            if( this.resizableLeft && gripBounds.Left.Contains( clientLocation ) )
            {
                m.Result = contentControl ? transparent : (IntPtr) NativeMethods.HTLEFT;
                return true;
            }
            if( !this.resizableLeft && gripBounds.Right.Contains( clientLocation ) )
            {
                m.Result = contentControl ? transparent : (IntPtr) NativeMethods.HTRIGHT;
                return true;
            }
            return false;
        }

        #endregion

        private System.Windows.Forms.VisualStyles.VisualStyleRenderer sizeGripRenderer;

        public void PaintSizeGrip( PaintEventArgs e )
        {
            if( e == null || e.Graphics == null || !this.resizable )
            {
                return;
            }

            Size clientSize = this.content.ClientSize;

            using( Bitmap gripImage = new Bitmap( 0x10, 0x10 ) )
            {
                using( Graphics g = Graphics.FromImage( gripImage ) )
                {
                    if( Application.RenderWithVisualStyles )
                    {
                        if( this.sizeGripRenderer == null )
                            this.sizeGripRenderer = new System.Windows.Forms.VisualStyles.VisualStyleRenderer( System.Windows.Forms.VisualStyles.VisualStyleElement.Status.Gripper.Normal );
                        this.sizeGripRenderer.DrawBackground( g, new Rectangle( 0, 0, 0x10, 0x10 ) );
                    }
                    else
                    {
                        ControlPaint.DrawSizeGrip( g, this.childPopup.BackColor, 0, 0, 0x10, 0x10 );
                    }
                }
                GraphicsState gs = e.Graphics.Save();
                e.Graphics.ResetTransform();
                if( this.resizableTop )
                {
                    if( this.resizableLeft )
                    {
                        e.Graphics.RotateTransform( 180 );
                        e.Graphics.TranslateTransform( -clientSize.Width, -clientSize.Height );
                    }
                    else
                    {
                        e.Graphics.ScaleTransform( 1, -1 );
                        e.Graphics.TranslateTransform( 0, -clientSize.Height );
                    }
                }
                else if( this.resizableLeft )
                {
                    e.Graphics.ScaleTransform( -1, 1 );
                    e.Graphics.TranslateTransform( -clientSize.Width, 0 );
                }
                e.Graphics.DrawImage( gripImage, clientSize.Width - 0x10, clientSize.Height - 0x10 );
                e.Graphics.Restore( gs );
            }
        }

        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if( this.content != null )
                {
                    this.content.Dispose();
                    this.content = null;
                }
                this.ownerPopup = null;
                this.childPopup = null;
            }
            base.Dispose( disposing );
        }

        #endregion

        #region Helper classes

        internal class GripBounds
        {
            private const int GripSize = 6;
            private const int CornerGripSize = GripSize << 1;

            public GripBounds( Rectangle clientRectangle )
            {
                this.clientRectangle = clientRectangle;
            }

            private Rectangle clientRectangle;
            public Rectangle ClientRectangle
            {
                get { return this.clientRectangle; }
            }

            public Rectangle Bottom
            {
                get
                {
                    Rectangle rect = this.clientRectangle;
                    rect.Y = rect.Bottom - GripSize + 1;
                    rect.Height = GripSize;
                    return rect;
                }
            }

            public Rectangle Left
            {
                get
                {
                    Rectangle rect = this.clientRectangle;
                    rect.Width = GripSize;
                    return rect;
                }
            }

            public Rectangle Right
            {
                get
                {
                    Rectangle rect = this.clientRectangle;
                    rect.X = rect.Right - GripSize + 1;
                    rect.Height = GripSize;
                    return rect;
                }
            }

            public Rectangle Top
            {
                get
                {
                    Rectangle rect = this.clientRectangle;
                    rect.Height = GripSize;
                    return rect;
                }
            }

            public Rectangle BottomLeft
            {
                get
                {
                    Rectangle rect = this.clientRectangle;
                    rect.Y = rect.Bottom - CornerGripSize + 1;
                    rect.Height = CornerGripSize;
                    rect.Width = CornerGripSize;
                    return rect;
                }
            }

            public Rectangle BottomRight
            {
                get
                {
                    Rectangle rect = this.clientRectangle;
                    rect.Y = rect.Bottom - CornerGripSize + 1;
                    rect.Height = CornerGripSize;
                    rect.X = rect.Right - CornerGripSize + 1;
                    rect.Width = CornerGripSize;
                    return rect;
                }
            }

            public Rectangle TopLeft
            {
                get
                {
                    Rectangle rect = this.clientRectangle;
                    rect.Height = CornerGripSize;
                    rect.Width = CornerGripSize;
                    return rect;
                }
            }

            public Rectangle TopRight
            {
                get
                {
                    Rectangle rect = this.clientRectangle;
                    rect.Height = CornerGripSize;
                    rect.X = rect.Right - CornerGripSize + 1;
                    rect.Width = CornerGripSize;
                    return rect;
                }
            }
        }
        #endregion
    }
}
