﻿// $Id: Script.cs 93 2012-11-12 19:07:22Z mawerick $
//
//  Copyright (C) Mawerick, WrongPlace.Net 2009 - 2012
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
using System;
using System.Collections.Generic;
using System.IO;

namespace Twp.FC
{
    /// <summary>
    /// Description of Script.
    /// </summary>
    public class Script : IEquatable<Script>, IComparable<Script>
    {
        public Script( FileInfo file )
        {
            this.file = file;
        }
        
        private FileInfo file;
        
        public bool IsValid()
        {
            return this.file.Exists;
        }
        
        public static bool IsValid( string path, string script )
        {
            return File.Exists( Path.Combine( path, script ) );
        }
        
        public void Create()
        {
            this.file.Create();
        }
        
        public static List<Script> List( string path )
		{
			if( String.IsNullOrEmpty( path ) )
				return null;

			DirectoryInfo di = new DirectoryInfo( Path.Combine( path, "scripts" ) );
			if( !di.Exists )
				return null;

			FileInfo[] files = di.GetFiles();
			if( files.Length > 0 )
			{
				List<Script> scripts = new List<Script>();
				foreach( FileInfo file in files )
				{
					Script script = new Script( file );
					scripts.Add( script );
				}
				return scripts;
			}
			else
				return null;
		}

		public string Name
		{
			get { return this.file.Name; }
		}
		
		private List<string> lines = null;
		public List<string> Lines
		{
		    get
		    {
		        if( this.lines == null )
		        {
		            this.ReadLines();
		        }
		        return this.lines;
		    }
		}
		
		private string content = null;
		public string Content
		{
		    get
		    {
		        if( this.content == null )
		        {
		            this.ReadContent();
		        }
		        return this.content;
		    }
		}
		
		public void ReadLines()
		{
            this.lines = new List<string>();
            using( StreamReader sr = new StreamReader( this.file.FullName ) )
            {
                while( !sr.EndOfStream )
                {
	                this.lines.Add( sr.ReadLine() );
                }
            }
		}
		
		public void ReadContent()
		{
            this.lines = new List<string>();
            using( StreamReader sr = new StreamReader( this.file.FullName ) )
            {
                this.content = sr.ReadToEnd();
            }
		}
		
		public void Save()
		{
		    using( StreamWriter sw = new StreamWriter( this.file.FullName, false ) )
		    {
		        foreach( string line in this.lines )
		        {
		            sw.WriteLine( line );
		        }
		        sw.Flush();
		        sw.Close();
		    }
		}

		public override string ToString()
		{
			if( String.IsNullOrEmpty( this.Name ) )
				return base.ToString();
			else
				return this.Name;
		}

		#region IEquatable implementation

		public override int GetHashCode()
		{
			return this.Name.GetHashCode();
		}

		public bool Equals( Script other )
		{
			if( other == null )
				return false;
			else
				return this.Name.Equals( other.Name );
		}

		public override bool Equals( object obj )
		{
			Script other = obj as Script;
			if( other != null )
				return this.Equals( other );
			else
				return false;
		}

		public static bool operator ==( Script script1, Script script2 )
		{
			if( object.ReferenceEquals( script1, script2 ) )
				return true;
			if( object.ReferenceEquals( script1, null ) )
				return false;
			if( object.ReferenceEquals( script2, null ) )
				return false;

			return script1.Equals( script2 );
		}

		public static bool operator !=( Script script1, Script script2 )
		{
			if( object.ReferenceEquals( script1, script2 ) )
				return false;
			if( object.ReferenceEquals( script1, null ) )
				return true;
			if( object.ReferenceEquals( script2, null ) )
				return true;

			return !script1.Equals( script2 );
		}

		#endregion

		#region IComparable implementation

		public int CompareTo( Script other )
		{
			return this.Name.CompareTo( other.Name );
		}

		#endregion
    }
}
